<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly.
}

final class CPB_Importer {
    public static function activate(): void {
        // Schedule the event on activation if auto-import is enabled
        if ( get_option( 'cpb_auto_import', false ) && ! wp_next_scheduled( 'cpb_import_rss_event' ) ) {
            wp_schedule_event( time(), 'weekly', 'cpb_import_rss_event' );
        }
    }

    public static function deactivate(): void {
        // Clear the scheduled event on deactivation
        wp_clear_scheduled_hook( 'cpb_import_rss_event' );
    }

    public static function import_rss() {
        $rss_feed = get_option( 'cpb_rss_feed', 'https://paraquemdoar.org/feed/' );
        $default_status = get_option( 'cpb_default_status', 'draft' );
        $import_from_date = get_option( 'cpb_import_from_date', '' );
        $selected_categories = get_option( 'cpb_import_categories', array() );

        // Load the RSS feed
        $rss = new DOMDocument();
        $rss->load($rss_feed);
        
        $xpath = new DOMXPath($rss);
        $xpath->registerNamespace('content', 'http://purl.org/rss/1.0/modules/content/');

        $items = $rss->getElementsByTagName('item');

        if ( ! $items->length ) {
            return new WP_Error( 'rss_error', 'Failed to load RSS feed.' );
        }

        $imported_posts = array();

        foreach ( $items as $item ) {
            $title = $item->getElementsByTagName('title')->item(0)->nodeValue;

            // Use WP_Query to check if the post already exists
            $query = new WP_Query(array(
                'title' => $title,
                'post_type' => 'post',
                'post_status' => 'any',
                'posts_per_page' => 1
            ));

            if ( $query->have_posts() ) {
                continue;
            }

            $description_node = $item->getElementsByTagName('description')->item(0);
            $content_encoded_node = $xpath->query('content:encoded', $item)->item(0);
            $image_node = $item->getElementsByTagName('image')->item(0);
            $pub_date_node = $item->getElementsByTagName('pubDate')->item(0);

            $description = $description_node ? $description_node->nodeValue : '';
            $content_encoded = $content_encoded_node ? $content_encoded_node->nodeValue : '';

            // Skipping posts before the user-selected date
            if ( ! empty( $import_from_date ) && $pub_date_node ) {
                $pub_date = strtotime( $pub_date_node->nodeValue );
                if ( $pub_date < strtotime( $import_from_date ) ) {
                    continue;
                }
            }

            // Concatenate description and content:encoded
            $post_content = $description . "\n" . $content_encoded;

            // Process and replace image URLs in the content
            $post_content = self::process_images_in_content($post_content);

            $post_data = array(
                'post_title' => $title,
                'post_content' => $post_content,
                'post_status' => $default_status,
                'post_author' => get_current_user_id(),
            );

            $post_id = wp_insert_post( $post_data );

            if ( is_wp_error( $post_id ) ) {
                $imported_posts[] = array(
                    'title' => $title,
                    'error' => $post_id->get_error_message(),
                );
            } else {
                // Set the post categories only if any selected
                if ( ! empty( $selected_categories ) ) {
                    wp_set_post_categories( $post_id, $selected_categories );
                }

                // Import image as featured image
                if ($image_node) {
                    $image_url = $image_node->nodeValue;
                    if ($image_url) {
                        $image_id = self::import_featured_image($image_url, $post_id);
                        if ( ! is_wp_error($image_id)) {
                            set_post_thumbnail($post_id, $image_id);
                        }
                    }
                }
                $imported_posts[] = array(
                    'title' => $title,
                    'error' => '',
                );
            }
        }

        return $imported_posts;
    }

    private static function process_images_in_content($content) {
        // Use DOMDocument to parse the HTML content
        $dom = new DOMDocument();
        @$dom->loadHTML($content);
        $images = $dom->getElementsByTagName('img');

        foreach ($images as $img) {
            $img_url = $img->getAttribute('src');
            $image_id = self::import_featured_image($img_url);
            if ( ! is_wp_error($image_id)) {
                // Get the URL of the imported image
                $new_img_url = wp_get_attachment_url($image_id);
                // Replace the old URL with the new URL in the content
                $content = str_replace($img_url, $new_img_url, $content);
            }
        }

        return $content;
    }

    private static function import_featured_image($image_url, $post_id = 0) {
        $upload_dir = wp_upload_dir();
        $image_data = file_get_contents($image_url);

        if (false === $image_data) {
            return new WP_Error('image_download_error', 'Failed to download image.');
        }

        $filename = basename($image_url);
        $file = $upload_dir['path'] . '/' . $filename;

        file_put_contents($file, $image_data);

        $wp_filetype = wp_check_filetype($filename, null);
        $attachment = array(
            'post_mime_type' => $wp_filetype['type'],
            'post_title' => sanitize_file_name($filename),
            'post_content' => '',
            'post_status' => 'inherit'
        );

        $attach_id = wp_insert_attachment($attachment, $file, $post_id);
        require_once ABSPATH . 'wp-admin/includes/image.php';
        $attach_data = wp_generate_attachment_metadata($attach_id, $file);
        wp_update_attachment_metadata($attach_id, $attach_data);

        return $attach_id;
    }
}